;;
;; util-list.scm
;;
;; Various utilities so that no external libraries are needed.
;;
;; ISC License
;;
;; Copyright 2023 Brmlab, z.s.
;; Dominik Pantůček <dominik.pantucek@trustica.cz>
;;
;; Permission to use, copy, modify, and/or distribute this software
;; for any purpose with or without fee is hereby granted, provided
;; that the above copyright notice and this permission notice appear
;; in all copies.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
;; WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
;; AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
;; CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
;; OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
;; NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
;; CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
;;

(declare (unit util-list))

(import duck)

(module*
 util-list
 #:doc ("This module implements basic list functionality which is common in
most scheme implementations.")
 (
  filter
  util-list-tests!
  )

 (import scheme
	 (chicken base)
	 (chicken io)
	 (chicken process)
	 testing)

 (define/doc (filter pred? lst)
   ("* ```pred?``` - procedure accepting any value and returning #t or #f
* ```lst``` - list to be filtered

Returns a list containing only elements matching given ```pred?```
predicate.")
   (let loop ((lst lst)
	      (res '()))
     (if (null? lst)
	 (reverse res)
	 (if (pred? (car lst))
	     (loop (cdr lst)
		   (cons (car lst) res))
	     (loop (cdr lst)
		   res)))))

 ;; Performs utils module self-tests.
 (define (util-list-tests!)
   (run-tests
    util-list
    (test-equal? filter (filter odd? '(1 2 3 4)) '(1 3))
    (test-equal? filter (filter odd? '(2 4)) '())
    ))

 )
